// DDS_I2C_2   Input "A0" : 0-5V tuning channel 1-6
//             Input "A1" : switch channel group "H"=1; "L"=2 
//             Input "A2" : switch frq "H"=direct; "L"=offset   

#include <Wire.h>
#include <LCD.h>
#include <Rotary.h>
#include <LiquidCrystal_I2C.h>  // F Malpartida's NewLiquidCrystal library

#define I2C_ADDR    0x40 >> 1 // I2C address PCF8574
//#define I2C_ADDR    0x70 >> 1 // I2C address PCF8574A ( not tested )

// LCD display pins
#define BACKLIGHT_PIN  7
#define En_pin  6
#define Rw_pin  5
#define Rs_pin  4
#define D4_pin  0
#define D5_pin  1
#define D6_pin  2
#define D7_pin  3

#define  LED_OFF  1
#define  LED_ON  0

// prototypes
LiquidCrystal_I2C
lcd(I2C_ADDR,En_pin,Rw_pin,Rs_pin,D4_pin,D5_pin,D6_pin,D7_pin);

// DDS
#define W_CLK 8   // AD9850 load clock pin (FCLK)
#define FQ_UD 9   // freq update pin (FQUD)
#define DATA 10   // serial data load pin (DATA)
#define RESET 12  // reset pin (RST) 

#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }

Rotary r = Rotary(2,3); // sets the pins the rotary encoder uses.  Must be interrupt pins.

int_fast32_t rx=14100000; // Starting frequency of VFO 14.100 MHz
int_fast32_t rx2=1; // variable to hold the updated frequency
int_fast32_t rx3=1; // variable to hold the offset frequency
int_fast32_t increment = 100; // VFO increment in Hz.

float frq_anz;
word channel_raw;
int  channel;
int  channel_old;
const int inputPin1 = A1;
const int inputPin2 = A2;

////////////////////////////////////////////////////////////
void setup() {    
  Serial.begin(9600);    
  pinMode(A0,INPUT); 

  lcd.begin (16,2);  // display init
  lcd.setBacklightPin(BACKLIGHT_PIN,NEGATIVE);
  lcd.setBacklight(LED_ON);
  lcd.backlight();    // Background light "ON"

  lcd.setCursor(0,0); // Char 1, Line 1
  lcd.print(" DDS-GENERATOR  ");
  lcd.setCursor(0,1); // Char 1, Line 2
  lcd.print(" (C)DJ7OO 2014  ");
  delay(3000);
  lcd.clear();

  lcd.setCursor(3,0); // Char 3, Line 1
  lcd.print("FREQUENCY:      ");

  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();
  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 
  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.

  digitalWrite(inputPin1,HIGH); 
  digitalWrite(inputPin2,HIGH);  
 }

////////////////////////////////////////////////////////////
void loop() {
 channel_raw = analogRead(A0);
// Serial.println(channel_raw);
 channel_raw = channel_raw/174;
 channel = channel_raw; 
// Serial.println(channel);

  if(channel != channel_old) 
  {
int val = digitalRead(inputPin1);
  if(val == HIGH) { 
// replace rx values according to your personal requirements
     if (channel==0) { rx=3600000; }  // 3.6 MHz
else if (channel==1) { rx=5300000; }  // 5.3 MHz
else if (channel==2) { rx=7100000; }  // 7.1 MHz
else if (channel==3) { rx=14100000; }  // 14.1 MHz
else if (channel==4) { rx=21100000; }  // 21.1 MHz
else if (channel==5) { rx=28500000; }  // 28.5 MHz
 channel_old = channel;  
  }
  else if(val== LOW) {
// replace rx values according to your personal requirements
     if (channel==0) { rx=3900000; }  // 3.9 MHz
else if (channel==1) { rx=6100000; }  // 6.1 MHz
else if (channel==2) { rx=7300000; }  // 7.3 MHz
else if (channel==3) { rx=9700000; }  // 9.7 MHz
else if (channel==4) { rx=11800000; }  // 11.8 MHz
else if (channel==5) { rx=15100000; }  // 15.1 MHz
 channel_old = channel;  
  }
}

  if (rx != rx2)  
  {    
  frq_anz = rx;
  frq_anz = frq_anz/1000000;
  rx2 = rx;

  lcd.setCursor(2,1); // Char 2, Line 2
  lcd.print(frq_anz,4);
  lcd.print(" MHz   ");

  Serial.print("FRx=  ");   
  Serial.println(rx);

 // frequency offset = "ON" if input "A2" = "LOW"  
  int val = digitalRead(inputPin2);
  if(val == HIGH) 
  { rx3 = rx; }
  else if(val == LOW) 
  { rx3 = rx + 5000000; } // Frequency offset in Hz ( change according to personal requirements )

  Serial.print("Fdds= ");   
  Serial.println(rx3);
  sendFrequency(rx3);
  }  
}  

////////////////////////////////////////////////////////////
ISR(PCINT2_vect) {
  unsigned char result = r.process();
  if (result) {    
    if (result == DIR_CW){rx=rx+increment;}
    else {rx=rx-increment;};       
  }
}

////////////////////////////////////////////////////////////
// frequency calc from datasheet page 8 = <sys clock> * <frequency tuning word>/2^32
void sendFrequency(double frequency) {  
  int32_t freq = frequency * 4294967295/125000000;  // note 125 MHz clock on AD9850.  You can make 'slight' tuning variations here by adjusting the clock frequency.
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for AD9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}

////////////////////////////////////////////////////////////
// transfers a byte, a bit at a time, LSB first to the AD9850 via serial DATA line
void tfr_byte(byte data) {
  for (int i=0; i<8; i++, data>>=1) 
  {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}


