// DDS_TFT_4   Input "A0" : 0-5V tuning channel 1-6
//             Input "A1" : switch channel group "H"=1; "L"=2 
//             Input "A2" : switch frq "H"=direct; "L"=offset   

#include <Rotary.h>
#include <SPI.h>
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library

// TFT
#define sclk 13
#define mosi 11
#define cs   5
#define rst  6  
#define dc   7

// DDS
#define W_CLK 8   // AD9850 load clock pin (FCLK)
#define FQ_UD 9   // freq update pin (FQUD)
#define DATA 10   // serial data load pin (DATA)
#define RESET 12  // reset pin (RST) 

#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }

Adafruit_ST7735 tft = Adafruit_ST7735(cs, dc, rst);

Rotary r = Rotary(2,3); // sets the pins the rotary encoder uses.  Must be interrupt pins.

int_fast32_t rx=14100000; // Starting frequency of VFO 14.100 MHz
int_fast32_t rx2=1; // variable to hold the updated frequency
int_fast32_t rx3=1; // variable to hold the offset frequency
int_fast32_t increment = 100; // VFO increment in Hz.

float frq_anz;
word channel_raw;
int  channel;
int  channel_old;
const int inputPin1 = A1;
const int inputPin2 = A2;

////////////////////////////////////////////////////////////
void setup() {    
  Serial.begin(9600);    
  pinMode(A0,INPUT); 

  tft.initR(INITR_BLACKTAB);   
  tft.setRotation(tft.getRotation() + 1); // rotate 90degs
  tft.setTextSize(2);    //1-8
  tft.fillScreen(ST7735_BLACK);
  tft.setCursor(15, 25);
  tft.setTextColor(ST7735_YELLOW);
//  tft.setTextWrap(true);
  tft.print("FREQUENCY:");

  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();
  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 
  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.

  digitalWrite(inputPin1,HIGH); 
  digitalWrite(inputPin2,HIGH); 
 }

////////////////////////////////////////////////////////////
void loop() {
 channel_raw = analogRead(A0);
// Serial.println(channel_raw);
 channel_raw = channel_raw/174;
 channel = channel_raw; 
// Serial.println(channel);

  if(channel != channel_old) 
  {
int val = digitalRead(inputPin1);
  if(val == HIGH) { 
// replace rx values according to your personal requirements
     if (channel==0) { rx=3600000; }  // 3.6 MHz
else if (channel==1) { rx=5300000; }  // 5.3 MHz
else if (channel==2) { rx=7100000; }  // 7.1 MHz
else if (channel==3) { rx=14100000; }  // 14.1 MHz
else if (channel==4) { rx=21100000; }  // 21.1 MHz
else if (channel==5) { rx=28500000; }  // 28.5 MHz
 channel_old = channel;  
  }
  else if(val== LOW) {
// replace rx values according to your personal requirements
     if (channel==0) { rx=3900000; }  // 3.9 MHz
else if (channel==1) { rx=6100000; }  // 6.1 MHz
else if (channel==2) { rx=7300000; }  // 7.3 MHz
else if (channel==3) { rx=9700000; }  // 9.7 MHz
else if (channel==4) { rx=11800000; }  // 11.8 MHz
else if (channel==5) { rx=15100000; }  // 15.1 MHz
 channel_old = channel;  
  }
}
  
  if (rx != rx2)  
  {    
  frq_anz = rx;
  frq_anz = frq_anz/1000000;
  rx2 = rx;

  tft.fillRect(5,50,140,20,ST7735_BLACK);   //partly screen erasing by writing box
  tft.setCursor(15, 55);
  tft.print(frq_anz,4 );
  tft.print(" MHz");

  Serial.print("FRx=  ");   
  Serial.println(rx);

 // frequency offset = "ON" if input "A2" = "LOW"  
  int val = digitalRead(inputPin2);
  if(val == HIGH) 
  { rx3 = rx; }
  else if(val == LOW) 
  { rx3 = rx + 5000000; } // Frequency offset in Hz ( change according to personal requirements )

  Serial.print("Fdds= ");   
  Serial.println(rx3);
  sendFrequency(rx);
  }  
}  

////////////////////////////////////////////////////////////
ISR(PCINT2_vect) {
  unsigned char result = r.process();
  if (result) {    
    if (result == DIR_CW){rx=rx+increment;}
    else {rx=rx-increment;};       
  }
}

////////////////////////////////////////////////////////////
// frequency calc from datasheet page 8 = <sys clock> * <frequency tuning word>/2^32
void sendFrequency(double frequency) {  
  int32_t freq = frequency * 4294967295/125000000;  // note 125 MHz clock on AD9850.  You can make 'slight' tuning variations here by adjusting the clock frequency.
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for AD9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}

////////////////////////////////////////////////////////////
// transfers a byte, a bit at a time, LSB first to the AD9850 via serial DATA line
void tfr_byte(byte data) {
  for (int i=0; i<8; i++, data>>=1) 
  {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}


